@@ -29,6 +29,7 @@
 #include "plotshell.h"
 #include "libguppi/barplot.h"
 #include "libguppi/xyplot.h"
+#include "libguppi/pieplot.h"
 #include "plotlist.h"
 #include "guppixmltags.h"
 #include "term.h"
@@ -57,6 +58,8 @@ static GnomeUIInfo file_menu[] = {
 static GnomeUIInfo plot_menu[] = {
   { GNOME_APP_UI_ITEM, N_("_Bar"), N_("Make a bar chart"), App::barplot_cb, NULL, NULL, \
     GNOME_APP_PIXMAP_NONE, 0, 0, (GdkModifierType) 0, NULL},
+  { GNOME_APP_UI_ITEM, N_("_Pie"), N_("Make a pie chart"), App::pieplot_cb, NULL, NULL, \
+    GNOME_APP_PIXMAP_NONE, 0, 0, (GdkModifierType) 0, NULL},
   { GNOME_APP_UI_ITEM, N_("_XY"), N_("Make an XY plot"), App::scatterplot_cb, NULL, NULL, \
     GNOME_APP_PIXMAP_NONE, 0, 0, (GdkModifierType) 0, NULL},
   GNOMEUIINFO_END
@@ -81,6 +84,8 @@ App::App()
 {
   app_ = gnome_app_new("guppi", _("Guppi - Main Window"));
 
+  guppi_setup_window(GTK_WINDOW(app_), "main_window");
+
   gnome_app_set_contents(GNOME_APP(app_), list_.widget());
 
   gnome_app_create_menus_with_data(GNOME_APP(app_), main_menu, this);
@@ -152,6 +157,21 @@ App::scatterplot()
   guppi_plot_shell_new(new XyPlot(guppi_datastore())); 
 }
 
+void 
+App::pieplot_cb(GtkWidget* w, gpointer data)
+{
+  App* a = static_cast<App*>(data);
+  
+  a->pieplot();
+}
+
+void
+App::pieplot()
+{
+  guppi_plot_shell_new(new PiePlot(guppi_datastore())); 
+}
+
+
 
 void 
 App::import_cb(GtkWidget* w, gpointer data)
