@@ -98,6 +98,16 @@ guppi_datastore()
           }
         ds->insert(d);
 
+        d = new ScalarData;
+        d->set_name("Pie Data");
+        z = 3;
+        while (z < 5)
+          {
+            d->add(z*3.3);
+            ++z;
+          }
+        ds->insert(d);
+
         d = new ScalarData;
         d->set_name("Empty Data 0");
         ds->insert(d);
