@@ -43,6 +43,8 @@
 
 #include <math.h>
 
+#include "guppi-math.h"
+
 enum {
   ARG_0
 };
@@ -438,13 +440,13 @@ guppi_scatter_render (GnomeCanvasItem *item,
             
               i2c.transform(p);
             
-            // This is sort of poorly thought-out and should be fixed.
-            // Need to be more precise.
+              // This is sort of poorly thought-out and should be fixed.
+              // Need to be more precise.
               cx2 = (int)rint(p.x);
               cy2 = (int)rint(p.y);
-            
-            // Convert to buffer indices - could be merged with affine 
-            //  transform?
+              
+              // Convert to buffer indices - could be merged with affine 
+              //  transform?
               cx2 -= buf->rect.x0;
               cy2 -= buf->rect.y0;
             
