@@ -71,6 +71,8 @@ XyLayer::XyLayer(LayerType type)
                                        "DataLayer");
       type_registry_->add_builtin_type(RelationLayer, TypeRegistry::INVALID_TYPE, 
                                        "RelationLayer");
+      type_registry_->add_builtin_type(PriceLayer, TypeRegistry::INVALID_TYPE, 
+                                       "PriceLayer");
     }
 
   type_registry_->ref();
@@ -678,4 +680,229 @@ XyRelation::generate_name() const
   return _("Untitled Relation");
 }
 
+//////////////////////////////////////////////
 
+
+XyPriceBars::XyPriceBars(DataStore* ds)
+  : XyLayer(XyLayer::PriceLayer), store_(ds), time_data_(0)
+{
+  int i = 0;
+  while (i < PriceTypeEnd)
+    {
+      price_data_[i] = 0;
+      ++i;
+    }
+}
+
+XyPriceBars::~XyPriceBars()
+{
+  int i = 0;
+  while (i < PriceTypeEnd)
+    {
+      release_data(price_data_[i]);
+      price_data_[i] = 0;
+      ++i;
+    }
+
+  release_data(time_data_);
+  time_data_ = 0;
+}
+
+void 
+XyPriceBars::set_price_data(PriceType which, Data* d)
+{
+  g_return_if_fail(which < PriceTypeEnd);
+  g_return_if_fail((d == 0) || (d->type() == Data::Scalar));
+
+  if (price_data_[which] != 0)
+    {
+      release_data(price_data_[which]);
+      price_data_[which] = 0;
+    }
+
+  if (d && d->type() == Data::Scalar)
+    {
+      price_data_[which] = d->cast_to_scalar();
+
+      grab_data(d);
+    }
+}
+
+ScalarData* 
+XyPriceBars::price_data(PriceType which)
+{
+  g_return_val_if_fail(which < PriceTypeEnd, 0);  
+
+  return price_data_[which];
+}
+  
+void 
+XyPriceBars::set_time_data(Data* d)
+{
+  if (time_data_ == d)
+    return;
+
+  if (time_data_ != 0)
+    {
+      release_data(time_data_);
+      time_data_ = 0;
+    }
+
+  if (d && d->type() == Data::Scalar)
+    {
+      time_data_ = d;
+      grab_data(d);
+    }
+}
+
+Data* 
+XyPriceBars::time_data()
+{
+  return time_data_;
+}
+
+guint 
+XyPriceBars::nbars() const
+{
+  // FIXME we could cache this like XyDataLayer does.
+  if (time_data_ != 0)
+    {
+      bool have_all = true;
+
+      int i = 0;
+      while (i < PriceTypeEnd)
+        {
+          if (price_data_[i] == 0)
+            {
+              have_all = false;
+              break;
+            }
+          
+          ++i;
+        }
+
+      if (have_all)
+        {
+          guint minsize = time_data_->size();
+          
+          i = 0;
+          
+          while (i < PriceTypeEnd)
+            {
+              minsize = MIN(minsize, price_data_[i]->size());
+              ++i;
+            }
+
+          return minsize;
+        }
+      else
+        return 0;
+    }
+  else 
+    return 0;
+}
+
+string 
+XyPriceBars::generate_name() const
+{
+  return "Price Bars"; // FIXME
+}
+
+void 
+XyPriceBars::add_data(DataStore* ds, Data* d)
+{
+  // Nothing
+  
+}
+
+
+void 
+XyPriceBars::remove_data(DataStore* ds, Data* d)
+{
+  // Remember that we can have multiple references to a single dataset
+
+  if (d == time_data_)
+    set_time_data(0);
+
+  int i = 0;
+  while (i < PriceTypeEnd)
+    {
+      if (price_data_[i] == (ScalarData*)d)
+        {
+          set_price_data((PriceType)i, 0);
+        }
+      
+      ++i;
+    }
+}
+
+
+void 
+XyPriceBars::change_data_values(DataStore* ds, 
+                                Data* d, 
+                                const vector<guint> & which)
+{  
+  if (!our_data(d))
+    return;
+
+  pricebars_model.data_values_changed(this, d);
+}
+
+void 
+XyPriceBars::change_data_values(DataStore* ds, Data* d)
+{
+  if (!our_data(d))
+    return;
+
+  // Number of points in d may have changed; check this if we ever cache
+  //  nbars
+
+  pricebars_model.data_values_changed(this, d);
+}
+
+void 
+XyPriceBars::change_data_name(DataStore* ds, Data* d, const string & name)
+{
+  // nothing
+}
+
+void 
+XyPriceBars::destroy_model()
+{
+  // nothing
+}
+
+void 
+XyPriceBars::grab_data(Data* d)
+{
+  if (d != 0)
+    d->ref();
+}
+
+void 
+XyPriceBars::release_data(Data* d)
+{
+  if (d != 0)
+    if (d->unref() == 0)
+      delete d;
+}
+
+bool 
+XyPriceBars::our_data(Data* d)
+{
+  if (d == time_data_)
+    return true;
+
+  int i = 0;
+  while (i < PriceTypeEnd)
+    {
+      if (price_data_[i] == (ScalarData*)d)
+        {
+          return true;
+        }
+      
+      ++i;
+    }
+  
+  return false;
+}
