@@ -28,7 +28,7 @@
 #include "scalardata.h"
 #include "typeregistry.h"
 
-#define OUR_PAD GNOME_PAD_SMALL
+#define OUR_PAD 3
 
 ////////// XyLayerEdit
 
@@ -40,7 +40,7 @@ public:
     : layer_(layer), state_(state), widget_(0), 
       layer_widget_(0), name_label_(0),
       x_axis_(0), y_axis_(0), 
-      hidden_toggle_(0), 
+      hidden_toggle_(0), masked_toggle_(0),
       remove_button_(0), raise_button_(0), lower_button_(0)
     {
       layer_->ref();
@@ -110,6 +110,7 @@ protected:
     x_axis_ = 0;
     y_axis_ = 0;
     hidden_toggle_ = 0;
+    masked_toggle_ = 0;
     remove_button_ = 0;
     raise_button_ = 0;
     lower_button_ = 0;
@@ -121,6 +122,7 @@ private:
   GtkWidget* x_axis_;
   GtkWidget* y_axis_;
   GtkWidget* hidden_toggle_;
+  GtkWidget* masked_toggle_;
   GtkWidget* remove_button_;
   GtkWidget* raise_button_;
   GtkWidget* lower_button_;
@@ -132,6 +134,7 @@ private:
                         GtkWidget** option);
 
   static void toggle_hidden_cb(GtkWidget* w, gpointer data);
+  static void toggle_masked_cb(GtkWidget* w, gpointer data);
   
   map<GtkWidget*,PlotUtil::PositionType> axis_items_;
 
@@ -293,6 +296,21 @@ XyLayerEdit::ensure_layer_widget()
 
   gtk_box_pack_start(GTK_BOX(vbox), hidden_toggle_, FALSE, FALSE, OUR_PAD);
 
+  /// 
+
+  masked_toggle_ = gtk_check_button_new_with_label(_("Mask Layer"));
+
+  guppi_set_tooltip(masked_toggle_, 
+                    _("Tools do not affect masked layers."));
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(masked_toggle_), layer_->masked());
+
+  gtk_signal_connect(GTK_OBJECT(masked_toggle_), "toggled",
+                     GTK_SIGNAL_FUNC(toggle_masked_cb),
+                     this);
+
+  gtk_box_pack_start(GTK_BOX(vbox), masked_toggle_, FALSE, FALSE, OUR_PAD);
+  
   ///
 
   gtk_container_set_border_width(GTK_CONTAINER(vbox), OUR_PAD);
@@ -312,6 +330,17 @@ XyLayerEdit::toggle_hidden_cb(GtkWidget* w, gpointer data)
   xle->layer_->set_hidden(new_state);
 }
 
+
+void 
+XyLayerEdit::toggle_masked_cb(GtkWidget* w, gpointer data)
+{
+  XyLayerEdit* xle = static_cast<XyLayerEdit*>(data);
+
+  bool new_state = (GTK_TOGGLE_BUTTON(xle->masked_toggle_)->active != FALSE);
+
+  xle->layer_->set_masked(new_state);
+}
+
 void 
 XyLayerEdit::activate_axisitem_cb(GtkWidget* w, gpointer data)
 {
@@ -413,7 +442,10 @@ XyLayerEdit::change_name(XyLayer* layer,
 void 
 XyLayerEdit::change_masked(XyLayer* layer, bool maskedness)
 {
-  // FIXME
+  bool old_state = (GTK_TOGGLE_BUTTON(hidden_toggle_)->active != FALSE);
+
+  if (maskedness != old_state)
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(masked_toggle_), layer_->masked());
 }
 
 void
@@ -434,6 +466,10 @@ public:
       changing_data_(false), 
       x_optimize_toggle_(0), y_optimize_toggle_(0)
     {
+      // Only let the user choose scalar data
+      x_option_.restrict_type(Data::Scalar);
+      y_option_.restrict_type(Data::Scalar);
+
       // We never disconnect since the DataOption dies with us.
       x_option_.connect(data_chosen_cb, this);
       y_option_.connect(data_chosen_cb, this);
@@ -777,7 +813,10 @@ XyScatterEdit::make_line_layer()
 
   layer->share_datapair_with(scatter_);
 
-  state_->add_layer(layer);
+  layer->set_x_axis(scatter_->x_axis());
+  layer->set_y_axis(scatter_->y_axis());
+
+  state_->add_layer_no_auto_axis(layer);
 }
 
 
