@@ -30,6 +30,8 @@
 #include "vfont.h"
 #include "typeregistry.h"
 
+#include "guppi-math.h"
+
 OneAxis::OneAxis()
   : axis_checked_out_(false)
 {
@@ -154,8 +156,10 @@ XyAxes::push_axis_bounds(vector<PlotUtil::PositionType>& axes,
 {
   g_return_if_fail(axes.size() == starts.size());
   g_return_if_fail(axes.size() == stops.size());
+  g_return_if_fail(!axes.empty());
 
-  // First fill in the changed axes
+  AxisBounds new_ab;
+  AxisBounds old_ab;
 
   guint i = 0;
 
@@ -165,35 +169,21 @@ XyAxes::push_axis_bounds(vector<PlotUtil::PositionType>& axes,
 
       g_return_if_fail(pos < PlotUtil::PositionTypeEnd);
 
-      Axis* a = axes_[pos]->checkout_axis();  
-      
-      if (a != 0)
-        {
-          a->set_start(starts[i]);
-          a->set_stop(stops[i]);
-          
-          axes_[pos]->checkin_axis(a);
-        }
-      else 
-        g_warning("Couldn't check out X axis to modify it!");
+      old_ab.starts[pos] = axes_[pos]->axis().start();
+      old_ab.stops[pos] = axes_[pos]->axis().stop();
 
-      ++i;
-    }
-  
-  // Now fetch and store the bounds of all four.
+      new_ab.starts[pos] = starts[i];
+      new_ab.stops[pos] = stops[i];
 
-  AxisBounds ab;
-
-  i = 0;
-  while (i < PlotUtil::PositionTypeEnd)
-    {
-      ab.starts[i] = axes_[i]->axis().start();
-      ab.stops[i] = axes_[i]->axis().stop();
+      old_ab.set_axes.push_back(pos);
+      new_ab.set_axes.push_back(pos);
 
       ++i;
     }
 
-  axis_bounds_stack_.push_back(ab);
+  set_axis_bounds(new_ab);
+
+  axis_bounds_stack_.push_back(old_ab);
 }
 
 guint 
@@ -289,21 +279,21 @@ XyAxes::set_one(PlotUtil::PositionType pos, OneAxis* oa)
 void 
 XyAxes::set_axis_bounds(const AxisBounds& ab)
 {
-  gint i = 0;
-
-  while (i < PlotUtil::PositionTypeEnd)
+  vector<PlotUtil::PositionType>::const_iterator i = ab.set_axes.begin();
+  
+  while (i != ab.set_axes.end())
     {
-      if (axes_[i]->axis().start() != ab.starts[i] ||
-          axes_[i]->axis().stop() != ab.stops[i])
+      if (axes_[*i]->axis().start() != ab.starts[*i] ||
+          axes_[*i]->axis().stop() != ab.stops[*i])
         {
-          Axis* a = axes_[i]->checkout_axis();  
+          Axis* a = axes_[*i]->checkout_axis();  
           
           if (a != 0)
             {
-              a->set_start(ab.starts[i]);
-              a->set_stop(ab.stops[i]);
-              
-              axes_[i]->checkin_axis(a);
+              a->set_start(ab.starts[*i]);
+              a->set_stop(ab.stops[*i]);
+
+              axes_[*i]->checkin_axis(a);
             }
           else 
             g_warning("Couldn't check out axis to modify it!");
@@ -492,6 +482,15 @@ XyPlotState::add_layer(XyLayer* layer)
   state_model.layer_added(this, layer);
 }
 
+void 
+XyPlotState::add_layer_no_auto_axis(XyLayer* layer)
+{
+  layers_.push_back(layer);
+  grab_layer(layer);
+
+  state_model.layer_added(this, layer);
+}
+
 void 
 XyPlotState::lower_layer(iterator i)
 {
